@@ -24,6 +24,7 @@
 #include <KDE/KConfig>
 #include <KDE/KConfigGroup>
 #include <KDE/KLocale>
+#include <kdeversion.h>
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QLibrary>
@@ -89,7 +90,10 @@ QtMdiDecorationButton::QtMdiDecorationButton(ButtonType type, KCommonDecoration
 
 void QtMdiDecorationButton::init()
 {
-	setAttribute(Qt::WA_PaintOnScreen, false);
+        const QtMdiDecoration *deco = (const QtMdiDecoration *) decoration();
+        if (deco->onScreen() >= 0) {
+            setAttribute(Qt::WA_PaintOnScreen, deco->onScreen() == 0 ? false : true);
+        }
 	setAttribute(Qt::WA_NoSystemBackground, true);
 	setAutoFillBackground(false);
 	setFocusPolicy(Qt::NoFocus);
@@ -113,19 +117,23 @@ void QtMdiDecorationButton::reset(unsigned long /*changed*/)
 void QtMdiDecorationButton::initStyleOption(QStyleOptionButton &opt)
 {
         const QtMdiDecoration *deco = (const QtMdiDecoration *) decoration();
+        const bool isActive = deco->isActive();
+        QFont font = deco->options()->font(isActive);
 
 	opt.init(this);
+        font.setPointSizeF(font.pointSizeF() * deco->symbolSize);
+        opt.fontMetrics = QFontMetrics(font);
 	if (isDown()) {
 		opt.state |= QStyle::State_Selected;
 	} else {
 		opt.state &= ~QStyle::State_Selected;
 	}
-	if (decoration()->isActive()) {
+	if (isActive) {
 		opt.state |= QStyle::State_Active;
 	} else {
 		opt.state &= ~QStyle::State_Active;
 	}
-        opt.palette.setColor(QPalette::Window, deco->options()->color(KCommonDecoration::ColorTitleBar, deco->isActive()));
+        opt.palette.setColor(QPalette::Window, deco->options()->color(KCommonDecoration::ColorTitleBar, isActive));
 }
 
 
@@ -135,7 +143,7 @@ void QtMdiDecorationButton::paintEvent(QPaintEvent */* event */)
 	QPainter painter(this);
 	QStyleOptionButton opt;
 	initStyleOption(opt);
-        const int iconSize = parentWidget()->style()->pixelMetric(QStyle::PM_SmallIconSize, &opt, this);
+        const int iconSize = deco->iconSize > 0 ? deco->iconSize : parentWidget()->style()->pixelMetric(QStyle::PM_SmallIconSize, &opt, this);
         QIcon::Mode iconMode = QIcon::Normal;
 
         if (isDown()) {
@@ -160,7 +168,7 @@ void QtMdiDecorationButton::paintEvent(QPaintEvent */* event */)
         opt.palette.setColor(QPalette::Text, deco->options()->color(KCommonDecoration::ColorFont, opt.state & QStyle::State_Active));
         const int shift = opt.state & QStyle::State_Selected ? 1 : 0;
         const bool useIcon = (sp == QStyle::SP_TitleBarMenuButton) && deco->useIcon();
-        const QIcon icon = useIcon ? deco->icon().pixmap(iconSize, iconSize) : parentWidget()->style()->standardIcon(sp, &opt, this);
+        const QIcon icon = useIcon ? deco->icon().pixmap(iconSize, iconSize, iconMode) : parentWidget()->style()->standardIcon(sp, &opt, this);
 	painter.setOpacity(opt.state & QStyle::State_Active ? 1.0 : 0.7);
 	icon.paint(&painter, rect().adjusted(shift, shift, shift, shift), Qt::AlignCenter, iconMode);
 }
@@ -232,22 +240,23 @@ int QtMdiDecoration::layoutMetric(LayoutMetric lm, bool respectWindowState, cons
     switch (lm) {
         case LM_BorderLeft:
         case LM_BorderRight:
-        case LM_BorderBottom:
             return border ? borderWidth : 0;
+        case LM_BorderBottom:
+            return border ? borderHeight : 0;
         case LM_TitleBorderLeft:
         case LM_TitleBorderRight:
-            return border ? 4 : 0;
+            return borderWidth > 0 && border ? 4 : 0;
         case LM_TitleEdgeLeft:
         case LM_TitleEdgeRight:
-            return border ? 8 : 0;
+            return borderWidth > 0 && border ? 8 : 0;
         case LM_TitleEdgeTop:
-            return border ? 2 : 0;
+            return borderWidth > 0 && border ? 2 : 0;
         case LM_TitleEdgeBottom:
             return 1;
         case LM_TitleHeight:
             return titleHeight + 2;
         case LM_ButtonHeight:
-            return border ? titleHeight : titleHeight + 3;
+            return borderWidth > 0 && border ? titleHeight : titleHeight + 3;
         case LM_ButtonWidth:
             return titleHeight;
         case LM_ButtonSpacing:
@@ -256,6 +265,13 @@ int QtMdiDecoration::layoutMetric(LayoutMetric lm, bool respectWindowState, cons
             return 2;
         case LM_ButtonMarginTop:
             return 0;
+#if KDE_IS_VERSION(4,3,0)
+        case LM_OuterPaddingLeft:
+        case LM_OuterPaddingTop:
+        case LM_OuterPaddingRight:
+        case LM_OuterPaddingBottom:
+            return 0;
+#endif
     }
     return KCommonDecoration::layoutMetric(lm, respectWindowState, button);
 }
@@ -270,6 +286,15 @@ KCommonDecorationButton *QtMdiDecoration::createButton(ButtonType type)
 void QtMdiDecoration::init()
 {
 	QWidget wid;
+        KConfig configFile(QLatin1String("kwinskulpturerc"));
+        KConfigGroup conf(&configFile, "General");
+        QString styleName = conf.readEntry("QtStyle", QString());
+        if (!styleName.isEmpty()) {
+            style = loadStyle(styleName);
+            if (!style) {
+                style = QStyleFactory::create(styleName);
+            }
+        }
         if (style) {
         	wid.setStyle(style);
         }
@@ -283,50 +308,61 @@ void QtMdiDecoration::init()
 		autoRaise = wid.style()->styleHint(QStyle::SH_TitleBar_AutoRaise, &option, 0);
 	} else {
 		borderWidth = 4;
-		titleHeight = 16;
+		titleHeight = 20;
 		noBorder = false;
 		autoRaise = false;
 	}
-	KConfig configFile(QLatin1String("kwinskulpturerc"));
-	KConfigGroup conf(&configFile, "General");
 	coloredFrame = conf.readEntry("UseTitleBarBorderColors", false);
 	contrastFrame = conf.readEntry("UseExtraContrastBorder", false);
         showIcon = conf.readEntry("UseApplicationIcon", true);
-	switch (options()->preferredBorderSize(factory())) {
-		case BorderTiny:		borderWidth = 2; break;
-		default:
-		case BorderNormal:		borderWidth = 3; break;
-		case BorderLarge:		borderWidth = 4; break;
-		case BorderVeryLarge:	borderWidth = 5; break;
-		case BorderHuge:		borderWidth = titleHeight / 2; break;
-		case BorderVeryHuge:	borderWidth = titleHeight * 3 / 4; break;
-		case BorderOversized:	borderWidth = titleHeight; break;
-	}
+        onlyActiveFrame = conf.readEntry("DrawOnlyActiveFrames", false);
+        paintOnScreen = -1;
+        if (conf.hasKey("PaintOnScreen")) {
+            paintOnScreen = conf.readEntry("PaintOnScreen", false) ? 1 : 0;
+        }
+        textShift = conf.readEntry("TextShift", 0);
+        textShiftMaximized = conf.readEntry("TextShiftMaximized", 0);
+        QChar c = QChar('l', 0);
+        QString align = conf.readEntry("TitleAlign", QString());
+        if (!align.isEmpty()) {
+            c = align.at(0).toLower();
+        }
+        switch (c.toAscii()) {
+            case 'c':
+                titleAlignment = Qt::AlignHCenter;
+                break;
+            case 'r':
+                titleAlignment = Qt::AlignRight;
+                break;
+            default:
+                titleAlignment = Qt::AlignLeft;
+                break;
+        }
+        centerFullWidth = conf.readEntry("CenterFullWidth", true);
+        iconSize = conf.readEntry("IconSize", -1);
+        symbolSize = conf.readEntry("SymbolSize", 1.0);
+        if (conf.readEntry("TitleHeight", -1) >= 0) {
+            titleHeight = QFontMetrics(options()->font(true)).height() + conf.readEntry("TitleHeight", 8) - 4;
+        }
+        if (conf.readEntry("BorderSize", -1) >= 0) {
+            borderWidth = conf.readEntry("BorderSize", borderWidth);
+        }
+        borderHeight = borderWidth;
+        if (conf.readEntry("BottomBorderSize", -1) >= 0) {
+            borderHeight = conf.readEntry("BottomBorderSize", borderWidth);
+        }
 	if (contrastFrame) {
 		borderWidth += 1;
+                borderHeight += 1;
 	}
 	KCommonDecoration::init();
 	if (wid.style()) widget()->setStyle(wid.style());
 	widget()->setAutoFillBackground(false);
 	widget()->setAttribute(Qt::WA_NoSystemBackground, true);
 	widget()->setAttribute(Qt::WA_OpaquePaintEvent, true);
-	widget()->setAttribute(Qt::WA_PaintOnScreen, false);
-//        widget()->window()->setAttribute(Qt::WA_NoSystemBackground, true);
-}
-
-
-static inline QColor blend_color(const QColor &c0, const QColor &c1, qreal blend)
-{
-	int b = int(0.5 + 256.0 * blend);
-	b = qMin(256, qMax(0, b));
-	QRgb rgba0 = c0.rgba();
-	QRgb rgba1 = c1.rgba();
-	return QColor(
-		qRed(rgba0) + (((qRed(rgba1) - qRed(rgba0)) * b) >> 8),
-		qGreen(rgba0) + (((qGreen(rgba1) - qGreen(rgba0)) * b) >> 8),
-		qBlue(rgba0) + (((qBlue(rgba1) - qBlue(rgba0)) * b) >> 8),
-		qAlpha(rgba0) + (((qAlpha(rgba1) - qAlpha(rgba0)) * b) >> 8)
-	);
+        if (paintOnScreen >= 0) {
+            widget()->setAttribute(Qt::WA_PaintOnScreen, paintOnScreen == 0 ? false : true);
+        }
 }
 
 
@@ -347,7 +383,7 @@ void QtMdiDecoration::paintEvent(QPaintEvent */*event */)
 		option.rect.adjust(4, 4, -4, -4);
 		option.rect.setHeight(titleHeight);
 	}
-	if (!border) {
+        if (!border || borderWidth == 0) {
 		option.rect.adjust(-6, 0, 6, 0);
 	}
 	option.text = QString();
@@ -358,25 +394,17 @@ void QtMdiDecoration::paintEvent(QPaintEvent */*event */)
 	option.titleBarFlags = 0;
 #endif
 //	option.palette = options()->palette(ColorTitleBar, isActive());
-	if (coloredFrame && border) {
+	if (coloredFrame && border && (!onlyActiveFrame || isActive())) {
 		painter.fillRect(widget()->rect(), options()->color(ColorTitleBar, isActive()));
-#if 0
-		QColor color0 = option.palette.color(QPalette::Window);
-		QColor color1 = options()->color(ColorTitleBar, isActive());
-		QRect r = widget()->rect();
-		for (int i = 0; i < borderWidth; ++i) {
-			painter.setPen(blend_color(color1, color0, i / (borderWidth - 1.0)));
-			painter.drawRect(r.adjusted(0, 0, -1, -1));
-			r.adjust(1, 1, -1, -1);
-		}
-#endif
 	} else {
 		painter.fillRect(widget()->rect(), option.palette.color(QPalette::Window));
+                if (borderHeight >= borderWidth + 2) {
+                    QPalette palette;
+                    palette.setCurrentColorGroup(QPalette::Disabled);
+                    qDrawShadeLine(&painter, 1, widget()->rect().bottom() + qMax(2, borderWidth) - borderHeight, widget()->rect().right(), widget()->rect().bottom() + qMax(borderWidth, 2) - borderHeight, palette);
+                }
 	}
 	QFont font = options()->font(isActive());
-//	font.setBold(false);
-//	font.setItalic(true);
-//	font.setPointSizeF(font.pointSizeF() / 1.19);
 	painter.setFont(font);
 	option.palette.setColor(QPalette::Highlight, options()->color(ColorTitleBar, true));
 	option.palette.setColor(QPalette::HighlightedText, options()->color(ColorFont, true));
@@ -384,21 +412,36 @@ void QtMdiDecoration::paintEvent(QPaintEvent */*event */)
 	option.palette.setColor(QPalette::WindowText, options()->color(ColorFont, false));
 //	option.rect = painter.style()->subControlRect(QStyle::CC_TitleBar, &option, QStyle::SC_TitleBarCloseButton, widget());
 	painter.drawComplexControl(QStyle::CC_TitleBar, option);
-//painter.save(); painter.setCompositionMode(QPainter::CompositionMode_Source); painter.fillRect(QRect(140, 10, 30, 10), QColor(200, 100, 200, 100)); painter.restore();
 	// draw the title caption
 	// TODO use MDI code
 	painter.save();
+        border = borderWidth > 0 && border;
 	int captionWidth = width() - buttonsLeftWidth() - buttonsRightWidth() - (border ? 16 : 4);
 	option.text = option.fontMetrics.elidedText(caption(), Qt::ElideMiddle, captionWidth);
 	QRect labelRect = QRect((border ? 8 : 2) + buttonsLeftWidth(), 0, captionWidth, titleHeight + 4);
+        Qt::Alignment alignment = titleAlignment;
+        if (titleAlignment & Qt::AlignHCenter && centerFullWidth) {
+            if (labelRect.left() > (width() - option.fontMetrics.boundingRect(option.text).width()) >> 1) {
+                alignment = Qt::AlignLeft;
+            } else if (labelRect.right() < (width() + option.fontMetrics.boundingRect(option.text).width()) >> 1) {
+                alignment = Qt::AlignRight;
+            } else {
+                labelRect = QRect(0, 0, width(), titleHeight + 4);
+            }
+        }
+        if (border) {
+            labelRect.adjust(0, -textShift, 0, -textShift);
+        } else {
+            labelRect.adjust(0, -textShiftMaximized, 0, -textShiftMaximized);
+        }
 	painter.setPen(QColor(0, 0, 0, 25));
-	painter.drawText(labelRect.adjusted(1, 1, 1, 1), Qt::AlignLeft | Qt::AlignVCenter | Qt::TextSingleLine, option.text);
+	painter.drawText(labelRect.adjusted(1, 1, 1, 1), alignment | Qt::AlignVCenter | Qt::TextSingleLine, option.text);
         QColor color = options()->color(ColorFont, isActive());
 	if (!(option.state & QStyle::State_Active)) {
              color.setAlpha((color.alpha() * 180) >> 8);
         }
 	painter.setPen(color);
-	painter.drawText(labelRect, Qt::AlignLeft | Qt::AlignVCenter | Qt::TextSingleLine, option.text);
+	painter.drawText(labelRect, alignment | Qt::AlignVCenter | Qt::TextSingleLine, option.text);
 	painter.restore();
 
 	// draw the frame
@@ -419,6 +462,9 @@ void QtMdiDecoration::paintEvent(QPaintEvent */*event */)
 			painter.fillRect(frameOptions.rect.adjusted(frameOptions.rect.width() - 1, 0, 0, 0), outerLineFill);
 			frameOptions.rect.adjust(1, 1, -1, -1);
 		}
+                QRegion region = widget()->rect();
+                region -= widget()->rect().adjusted(borderWidth, borderWidth, -borderWidth, -borderWidth);
+                painter.setClipRegion(region);
 		painter.drawPrimitive(QStyle::PE_FrameWindow, frameOptions);
 	}
 }
@@ -451,19 +497,6 @@ bool SkulptureDecorationFactory::reset(unsigned long changed)
 }
 
 
-QList<SkulptureDecorationFactory::BorderSize> SkulptureDecorationFactory::borderSizes() const
-{
-	return QList<BorderSize>()
-			<< BorderTiny
-			<< BorderNormal
-			<< BorderLarge
-			<< BorderVeryLarge
-			<< BorderHuge
-			<< BorderVeryHuge
-			<< BorderOversized;
-}
-
-
 bool SkulptureDecorationFactory::supports(Ability ability) const
 {
 	switch (ability)
